"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenApiValidator = exports.Forbidden = exports.Unauthorized = exports.NotFound = exports.MethodNotAllowed = exports.BadRequest = exports.RequestEntityToLarge = exports.UnsupportedMediaType = exports.InternalServerError = void 0;
const ono_1 = require("ono");
const express = require("express");
const _uniq = require("lodash.uniq");
const middlewares = require("./middlewares");
const openapi_context_1 = require("./framework/openapi.context");
const resolvers_1 = require("./resolvers");
var types_1 = require("./framework/types");
Object.defineProperty(exports, "InternalServerError", { enumerable: true, get: function () { return types_1.InternalServerError; } });
Object.defineProperty(exports, "UnsupportedMediaType", { enumerable: true, get: function () { return types_1.UnsupportedMediaType; } });
Object.defineProperty(exports, "RequestEntityToLarge", { enumerable: true, get: function () { return types_1.RequestEntityToLarge; } });
Object.defineProperty(exports, "BadRequest", { enumerable: true, get: function () { return types_1.BadRequest; } });
Object.defineProperty(exports, "MethodNotAllowed", { enumerable: true, get: function () { return types_1.MethodNotAllowed; } });
Object.defineProperty(exports, "NotFound", { enumerable: true, get: function () { return types_1.NotFound; } });
Object.defineProperty(exports, "Unauthorized", { enumerable: true, get: function () { return types_1.Unauthorized; } });
Object.defineProperty(exports, "Forbidden", { enumerable: true, get: function () { return types_1.Forbidden; } });
class OpenApiValidator {
    constructor(options) {
        this.validateOptions(options);
        this.normalizeOptions(options);
        if (options.unknownFormats == null)
            options.unknownFormats === true;
        if (options.coerceTypes == null)
            options.coerceTypes = true;
        if (options.validateRequests == null)
            options.validateRequests = true;
        if (options.validateResponses == null)
            options.validateResponses = false;
        if (options.validateSecurity == null)
            options.validateSecurity = true;
        if (options.fileUploader == null)
            options.fileUploader = {};
        if (options.$refParser == null)
            options.$refParser = { mode: 'bundle' };
        if (options.validateFormats == null)
            options.validateFormats = 'fast';
        if (typeof options.operationHandlers === 'string') {
            /**
             * Internally, we want to convert this to a value typed OperationHandlerOptions.
             * In this way, we can treat the value as such when we go to install (rather than
             * re-interpreting it over and over).
             */
            options.operationHandlers = {
                basePath: options.operationHandlers,
                resolver: resolvers_1.defaultResolver,
            };
        }
        else if (typeof options.operationHandlers !== 'object') {
            // This covers cases where operationHandlers is null, undefined or false.
            options.operationHandlers = false;
        }
        if (options.validateResponses === true) {
            options.validateResponses = {
                removeAdditional: false,
            };
        }
        if (options.validateRequests === true) {
            options.validateRequests = {
                allowUnknownQueryParameters: false,
            };
        }
        if (options.validateSecurity === true) {
            options.validateSecurity = {};
        }
        this.options = options;
    }
    installMiddleware(spec) {
        const middlewares = [];
        const pContext = spec.then((spec) => new openapi_context_1.OpenApiContext(spec, this.options.ignorePaths));
        let inited = false;
        // install path params
        middlewares.push((req, res, next) => pContext
            .then((context) => {
            if (!inited)
                this.installPathParams(req.route || req.app, context);
            next();
        })
            .catch(next));
        // metadata middleware
        middlewares.push((req, res, next) => pContext
            .then((context) => this.metadataMiddlware(context)(req, res, next))
            .catch(next));
        if (this.options.fileUploader) {
            // multipart middleware
            middlewares.push((req, res, next) => pContext
                .then((context) => this.multipartMiddleware(context)(req, res, next))
                .catch(next));
        }
        middlewares.push((req, res, next) => pContext
            .then((context) => {
            const components = context.apiDoc.components;
            if (this.options.validateSecurity && (components === null || components === void 0 ? void 0 : components.securitySchemes)) {
                return this.securityMiddleware(context)(req, res, next);
            }
            else {
                next();
            }
        })
            .catch(next));
        if (this.options.validateRequests) {
            middlewares.push((req, res, next) => {
                return pContext
                    .then((context) => this.requestValidationMiddleware(context)(req, res, next))
                    .catch(next);
            });
        }
        if (this.options.validateResponses) {
            middlewares.push((req, res, next) => pContext
                .then((context) => this.responseValidationMiddleware(context)(req, res, next))
                .catch(next));
        }
        if (this.options.operationHandlers) {
            let router = null;
            middlewares.push((req, res, next) => {
                if (router)
                    return router(req, res, next);
                pContext
                    .then((context) => router = this.installOperationHandlers(req.baseUrl, context))
                    .then((router) => router(req, res, next)).catch(next);
            });
        }
        return middlewares;
    }
    installPathParams(app, context) {
        const pathParams = [];
        for (const route of context.routes) {
            if (route.pathParams.length > 0) {
                pathParams.push(...route.pathParams);
            }
        }
        // install param on routes with paths
        for (const p of _uniq(pathParams)) {
            app.param(p, (req, res, next, value, name) => {
                const openapi = req.openapi;
                if (openapi === null || openapi === void 0 ? void 0 : openapi.pathParams) {
                    const { pathParams } = openapi;
                    // override path params
                    req.params[name] = pathParams[name] || req.params[name];
                }
                next();
            });
        }
    }
    metadataMiddlware(context) {
        return middlewares.applyOpenApiMetadata(context);
    }
    multipartMiddleware(context) {
        return middlewares.multipart(context, {
            multerOpts: this.options.fileUploader,
            unknownFormats: this.options.unknownFormats,
        });
    }
    securityMiddleware(context) {
        var _a;
        const securityHandlers = (_a = (this.options.validateSecurity)) === null || _a === void 0 ? void 0 : _a.handlers;
        return middlewares.security(context, securityHandlers);
    }
    requestValidationMiddleware(context) {
        const { coerceTypes, unknownFormats, validateRequests, validateFormats, } = this.options;
        const { allowUnknownQueryParameters } = (validateRequests);
        const requestValidator = new middlewares.RequestValidator(context.apiDoc, {
            nullable: true,
            coerceTypes,
            removeAdditional: false,
            useDefaults: true,
            unknownFormats,
            allowUnknownQueryParameters,
            format: validateFormats,
        });
        return (req, res, next) => requestValidator.validate(req, res, next);
    }
    responseValidationMiddleware(context) {
        const { coerceTypes, unknownFormats, validateResponses, validateFormats, } = this.options;
        const { removeAdditional } = validateResponses;
        return new middlewares.ResponseValidator(context.apiDoc, {
            nullable: true,
            coerceTypes,
            removeAdditional,
            unknownFormats,
            format: validateFormats,
        }).validate();
    }
    installOperationHandlers(baseUrl, context) {
        const router = express.Router({ mergeParams: true });
        this.installPathParams(router, context);
        for (const route of context.routes) {
            const { method, expressRoute } = route;
            /**
             * This if-statement is here to "narrow" the type of options.operationHandlers
             * to OperationHandlerOptions (down from string | false | OperationHandlerOptions)
             * At this point of execution it _should_ be impossible for this to NOT be the correct
             * type as we re-assign during construction to verify this.
             */
            if (this.isOperationHandlerOptions(this.options.operationHandlers)) {
                const { basePath, resolver } = this.options.operationHandlers;
                const path = expressRoute.indexOf(baseUrl) === 0
                    ? expressRoute.substring(baseUrl.length)
                    : expressRoute;
                router[method.toLowerCase()](path, resolver(basePath, route));
            }
        }
        return router;
    }
    validateOptions(options) {
        if (!options.apiSpec)
            throw ono_1.default('apiSpec required');
        const securityHandlers = options.securityHandlers;
        if (securityHandlers != null) {
            throw ono_1.default('securityHandlers is not supported. Use validateSecurities.handlers instead.');
        }
        const multerOpts = options.multerOpts;
        if (multerOpts != null) {
            throw ono_1.default('multerOpts is not supported. Use fileUploader instead.');
        }
        const unknownFormats = options.unknownFormats;
        if (typeof unknownFormats === 'boolean') {
            if (!unknownFormats) {
                throw ono_1.default("unknownFormats must contain an array of unknownFormats, 'ignore' or true");
            }
        }
        else if (typeof unknownFormats === 'string' &&
            unknownFormats !== 'ignore' &&
            !Array.isArray(unknownFormats))
            throw ono_1.default("unknownFormats must contain an array of unknownFormats, 'ignore' or true");
    }
    normalizeOptions(options) {
        // Modify the request
    }
    isOperationHandlerOptions(value) {
        if (value.resolver) {
            return true;
        }
        else {
            return false;
        }
    }
}
exports.OpenApiValidator = OpenApiValidator;
//# sourceMappingURL=openapi.validator.js.map